//
// THIS FILE IS COPYRIGHT 1994, 1995 BY TELENOR R&D, NORWAY
//
//                        ALL RIGHTS RESERVED.
//
// For rights granted to the public see the COPYRIGHT file
//
//
// main.cc	The main routine for the FFW index program.
//
// Author:  Ken Ronny Schouten
//
// Changes: January 1995 Brd Hfjeld
//				Added -f option to support dates from filesys
//	    June 1995 Brd Hfjeld
//				Added options -n, -c, -l, -u to support
//				more filter functionality
//	    August 1995 Brd Hfjeld
//				Added -m option
// 	    September 1995 Brd Hfjeld
//				Added support for filenames with blanks
//				in parameter files.
//
#include <stream.h>
#include <String.h>
#include <stdlib.h>
#include <GetOpt.h>
#include <sys/types.h>
#include <unistd.h>
#include <Index.h>
#include <Html.h>
#include <Disk.h>
#include <S.SplaySet.h>
#include <lang.h>
#include <Wordfilter.h>
#include <ctype.h>

// Proto types
int  CheckStdin(int *, char **, int);

// default filter params
WordFilterParams filter_params = {
    0,		// minimum permissible word length
    32767,	// maximum permissible word length
    1,		// allow numbers
    1,		// allow strings with all identical chars
};

#define True (1)
#define False (0)

// Oh glorious main, thou shall never die 
// ..
// Parse the command line for input.
// First parameter should be a file name that gives you 
// a name for the index file you shall store in. Second 
// should be a URL-root for the document you are going to index.

int main(int argc, char **argv)
{
    StringSplaySet Ifiles;
    StringSplaySet Wfiles;
    GetOpt getopt(argc, argv, "fi:w:ncl:u:m");
    int c;
    int override = 0;
    int std_in;
    Index index;
    Wordfilter *filter;
    int html_options = 0;
    String tmpstr;

    // Check that there is at least 1 parameters
    if (argc < 2) {
	cout.form(UsageIndex, argv[0]);
	exit(1);
    }

    // Set up time system to be GMT for mktime()
    putenv("TZ=GMT");
    tzset();

    while ((c = getopt()) != EOF) {
	switch (c) {
	case 'f':
	    html_options |= HOPT_FILEDATE;
	    break;
	case 'i':
	    tmpstr = getopt.optarg;
	    Ifiles.add(tmpstr);
	    break;
	case 'w':
	    tmpstr = getopt.optarg;
	    Wfiles.add(tmpstr);
	    break;
	case 'n':
	    filter_params.allow_numbers = False;
	    break;
	case 'c':
	    filter_params.allow_identicals = False;
	    break;
	case 'l':
	    filter_params.min_word_length = atoi(getopt.optarg);
	    break;
	case 'u':
	    filter_params.max_word_length = atoi(getopt.optarg);
	    break;
	case 'm':
	    html_options |= HOPT_COMMENTS;
	    break;
	}
    }
    
    std_in = CheckStdin(&argc, argv, getopt.optind);

    if (getopt.optind == 0)
	getopt.optind = 1;

    if (argc - getopt.optind < 1) {
	cout.form (UsageIndex, argv[0]);
	exit(1);
    }

    unsigned char *index_name = (unsigned char *)
		strdup(argv[getopt.optind++]);

    // Set up the filter
    filter = new Wordfilter(filter_params);
    for (Pix y = Wfiles.first() ; y != 0 ;Wfiles.next(y)) {	
	filter->add(Wfiles(y));
    }

    unsigned char *url_root = NULL;
    if (getopt.optind < argc) {
	url_root = (unsigned char *) strdup(argv[getopt.optind++]);
    }

    // Process file names from the command line
    for (int i = getopt.optind; i < argc; i++) {
	Html html((char *) argv[i], (char *)NULL, (char *) url_root,
	      	  *filter, index, html_options);

	if (html.error() != 0) {
	    switch (html.error()) {
	    case HOPEN:
		cout.form(OpenFile, argv[i]);
		break;
	    default:
		cout.form(UnknownError, html.error(), "Html::Html()");
		break;
	    }
	} else {
	    cout << "."; cout.flush();
	}
    }

    // Process parameter files
    for (Pix x = Ifiles.first() ; x != 0 ;Ifiles.next(x)) {
	filebuf fin;

	if ((fin.open(Ifiles(x), input)) == 0) {
	    cerr.form(OpenFile, (char *) Ifiles(x));
	}
	istream file(&fin);

	unsigned char in_line[200];
	unsigned char url[200];
	unsigned char dir[200];
	if ( url_root )
	    strncpy((char *)url, (char *) url_root, sizeof(url)-1);
	else *url = '\0';
	*dir = '\0';
    
	file.getline(in_line, sizeof(in_line));
	while ( file && *in_line=='\0') file.getline(in_line, sizeof(in_line));

	while (file) {
	    if ( strncmp8((unsigned char *)"NEWPARAMETERS", in_line, 13)==0 ) {
		unsigned char *cp = in_line + 13;
		unsigned char *up, *dp;
		while ( *cp && isspace(*cp) ) cp++;
		up = cp;
		while ( *cp && !isspace(*cp) ) cp++;
		if ( *cp ) *(cp++) = '\0';
		while ( *cp && isspace(*cp) ) cp++;
		dp = cp;
		while ( *cp && !isspace(*cp) ) cp++;
		if ( *cp ) *(cp++) = '\0';
		strncpy((char *)url, (char *)up, sizeof(url)-1);
		strncpy((char *)dir, (char *)dp, sizeof(dir)-1);
	    } else {
		// string shows the file to be opened.
		Html html((char *)in_line, (char *)dir, (char *)url,
			  *filter, index, html_options);

		if (html.error() != 0) {
	    	    switch (html.error()) {
	    		case HOPEN:
			    cout.form(OpenFile, (char *)in_line);
			    break;
	    		default:
			    cout.form(UnknownError, html.error(),
				      "Html::Html()");
			    break;
	    	    }
		} else {
	    	    cout << "."; cout.flush();
		}
	    }
	    file.getline(in_line, sizeof(in_line));
	    while ( file && *in_line=='\0')
		file.getline(in_line, sizeof(in_line));
	}
    }

    // Process filenames from stdin
    if (std_in) {
	unsigned char in_line[200];
	unsigned char url[200];
	unsigned char dir[200];
	if ( url_root )
	    strncpy((char *)url, (char *) url_root, sizeof(url)-1);
	else *url = '\0';
	*dir = '\0';
    
	cin.getline(in_line, sizeof(in_line));
	while ( cin && *in_line=='\0') cin.getline(in_line, sizeof(in_line));
	
	while (cin) {
	    if ( strncmp8((unsigned char *)"NEWPARAMETERS", in_line, 13)==0 ) {
		unsigned char *cp = in_line + 13;
		unsigned char *up, *dp;
		while ( *cp && isspace(*cp) ) cp++;
		up = cp;
		while ( *cp && !isspace(*cp) ) cp++;
		if ( *cp ) *(cp++) = '\0';
		while ( *cp && isspace(*cp) ) cp++;
		dp = cp;
		while ( *cp && !isspace(*cp) ) cp++;
		if ( *cp ) *(cp++) = '\0';
		strncpy((char *)url, (char *)up, sizeof(url)-1);
		strncpy((char *)dir, (char *)dp, sizeof(dir)-1);
	    } else {
		// string shows the file to be opened.
		Html html((char *)in_line, (char *)dir, (char *)url,
			  *filter, index, html_options);

		if (html.error() != 0) {
	    	    switch (html.error()) {
	    		case HOPEN:
			    cout.form(OpenFile, (char *)in_line);
		 	    break;
	    		default:
			    cout.form(UnknownError, html.error(),
				      "Html::Html()");
			    break;
	    	    }
		} else {
	    	    cout << "."; cout.flush();
		}
	    }
	    cin.getline(in_line, sizeof(in_line));
	    while ( cin && *in_line=='\0')
		cin.getline(in_line, sizeof(in_line));
	}
    }

    cout << endl;

    Disk disk(index_name, DWRITE);
    if ((disk.error()) == DOPEN) {
	cout.form(OpenFile, (char *) index_name);
	exit(1);
    }

    if (disk.MakeIndexOnDisk(index) == -1) {
	switch ((disk.error())) {
	case DMODE:
	    cout << "Wrong mode." << endl;
	    break;
	default:
	    cout.form(UnknownError, disk.error(), "Disk::Disk()");
	    break;
	}
    }

    exit(0);
}

int CheckStdin(int *argc, char **argv, int start)
{
    int where = start;
    int value = 0;
    char *str;

    for (int i = start ; i < *argc ; i++, where++) {
	if (strcmp(argv[i],"-") == 0 && strlen(argv[i]) == 1) {
	    value = 1;
	    where--;
	}  else {
	    str = strdup(argv[i]);
	    argv[where] = str;
	}
    }
    
    *argc = *argc - (*argc - where);
    return value;
}
