//
// THIS FILE IS COPYRIGHT 1994, 1995 BY TELENOR R&D, NORWAY
//
//                        ALL RIGHTS RESERVED.
//
// For rights granted to the public see the COPYRIGHT file
//
//
// Entry.h	Entry class. Contains word with urllist.
//
// Author:  Ken Ronny Schouten

#ifndef _Entry_H
#define _Entry_H

#include <stdlib.h>
#include <string.h>
#include "UP.SplaySet.h"

extern int strcmp8(const unsigned char *s1, const unsigned char *s2);

class Entry {
public:
    unsigned char *_word;
    URLPTRSplaySet  _urllist;
    operator==(Entry& e);
    operator<=(Entry& e);
    operator<(Entry& e);
    operator>(Entry& e);
    Entry();
    Entry(unsigned char *s);
    Entry(const Entry& e);
    ~Entry();
    void deleteUrl();
    unsigned char *word();
    unsigned char first();
};

inline Entry::operator==(Entry& e)
{ 
    return strcmp8(_word, e._word) == 0;
}

inline Entry::operator<=(Entry& e) { 
    return strcmp8(_word, e._word) <= 0;
}

inline Entry::operator<(Entry& e) { 
    return strcmp8(_word, e._word) < 0;
}

inline Entry::operator>(Entry& e) { 
    return strcmp8(_word, e._word) > 0;
}

inline Entry::Entry() 
{ 
    _word = NULL; 
}

inline Entry::Entry(unsigned char *s) 
{ 
    _word = (unsigned char *) strdup((char *)s);
}

inline Entry::Entry(const Entry& e) 
{
    _word = (unsigned char *) strdup((char *)e._word);
    _urllist = e._urllist; 
}

inline Entry::~Entry() 
{ 
    free(_word); 
    _word = NULL; 
}

inline void Entry::deleteUrl() 
{
    Pix _ind_;
    for (_ind_ = _urllist.first(); _ind_ != 0; _urllist.next(_ind_)) {
	Url *url = _urllist(_ind_);
	_urllist.del(url);
	delete url;
    }
}

inline unsigned char *Entry::word() 
{ 
    return _word; 
}

inline unsigned char Entry::first() 
{ 
    return _word[0]; 
}

#endif /* _Entry_H */
