//
// THIS FILE IS COPYRIGHT 1994, 1995 BY TELENOR R&D, NORWAY
//
//                        ALL RIGHTS RESERVED.
//
// For rights granted to the public see the COPYRIGHT file
//
//
// Html.cc	Parses the html documents and adds all words to the
// 		global word list with an url.
//
// Author:  Ken Ronny Schouten
//
// Changes: January 95 Brd Hfjeld
//				Added document date mechanism
//				Removed bug in character entity handling
//	    March 1995 Brd Hfjeld
//				Added &#ddd; escape code handling by
//				Matt Hefron
//
#include <string.h>
#include <fstream.h>
#include <iostream.h> 
#include <stdlib.h> 
#include <String.h>
#include "Html.h"
#include "convert.h"
#include <time.h>
#include <ctype.h>
#include "lang.h"
#include <sys/types.h>
#include <sys/stat.h>
#include "div.h"

// This is the constructor of the HTML class. Here we will read in the
// html-document and parse it ready for the the index class. We will try to
// find the title field.
Html::Html(String filename, String dir, String urlroot, Wordfilter& filter,
	   Index& index, int options)
{
    short              stat = 0;
    unsigned char      word[MAX_LENGTH+1];
    unsigned char      title[MAX_LENGTH+1];
    unsigned char      special[MAX_SPECIAL+2];
    unsigned char      command[MAX_LENGTH+1];
    int                cmd_ind, title_ind = 0, spec_ind, word_ind = 0;

    unsigned char  buffer[MAX_BUFFER];

    _error = 0;

    if (strncmp(filename,"./", 2) == 0)
	filename = &filename[2]; 
    
    if (dir == NULL)
	dir = filename;
    else if (dir.length() > 0) {
	if (dir[dir.length()-1] != '/') 
	    dir += '/';
	dir += filename;
    }

    ifstream   html_read(dir);

    if (!html_read) {
	_error = HOPEN;
	return;
    }

    // Append name to urlroot, add / if needed
    if (urlroot.length() > 0) 
	if ( urlroot[urlroot.length()-1] != '/') 
	    urlroot += '/';
    urlroot += filename;

    // Create a new url for this file
    Url* url = new Url((unsigned char *)(char *) urlroot);

    // Insert the url into the global url list
    Pix n = index._url.add(url);

    // If the url already exists we got an error - two docs with same url
    // This version allows doubles for debug
    if ((void *) url != (void *) index._url(n)) {
	delete url;
	// return and give message to user
	url = index._url(n);
    }

    // Set default date from file if desired, else defaults to 0
    if ( options & HOPT_FILEDATE ) {
	struct stat stats;
	if ( fstat(html_read.rdbuf()->fd(), &stats) ) {
	    // Hmm.. silently ignore errors here
	} else {
	    url->setDate(stats.st_mtime);
	}
    }

    
    while (html_read) {
	// Hmm, we should test if this read is returning an error. If so we 
	// should tell the user of the Html object.
	html_read.read(buffer, MAX_BUFFER-1);
	buffer[html_read.gcount()] = '\0';
	
	for (unsigned char *str = buffer; *str ; str++) {
	    // Test for special html commands like '<' and '&'.
	    // < found in escapes is handled further down to try
	    // recover the tag
	    if ( !BIT(stat, SPECIAL) && *str == '<') {
		SET_BIT(stat, READ_COMMAND);
		cmd_ind = 0;
		continue;
	    }

	    // End of a HTML tag
	    if (*str == '>' && BIT(stat, READ_COMMAND)) {
		CLR_BIT(stat, READ_COMMAND);
		command[cmd_ind] = '\0';
		parseHtmlCommand(command, &stat, index._url(n),
				 filter, index, options);
		continue;
	    }

	    // Beginning of an escape code
	    if (*str == '&') {
	       	SET_BIT(stat, SPECIAL);
		spec_ind = 0;
		continue;
	    }

	    // Reading an escape code
	    if (BIT(stat, SPECIAL)) {
		if (*str != ';'  && *str != ' ' && *str != '\n' &&
		    *str != '<') {
		    // Collecting escape code name
		    special[spec_ind++] = *str;

		    // If length overflow for escape code, ignore it
		    if ( spec_ind > MAX_SPECIAL ) {
		    	CLR_BIT(stat, SPECIAL);
		    	spec_ind = 0;
		    }
		    
		    continue;
		} else {
		    // End of escape code

		    // Try protect tags from unterminated escapes
		    if ( *str == '<' && str>buffer ) str--;

		    int i = 0;
		    special[spec_ind] = '\0';

		    //[BII] Parse &#ddd; form of escape code...
 		    if (special[0] == '#') {
 			char * pst;
 			i = (int) strtol((const char *)&special[1], &pst, 10);
 			if (!*pst) {
 			    *str = (char) i;
 			    CLR_BIT(stat, SPECIAL);
 			} else {
 			    i = NUM_SPECIAL;	// force the error, below
 			}
 		    } else { // Symbolic escapes
		    	while (i < NUM_SPECIAL) {
			    if (strcmp8(Trans[i].string, special) == 0) {
			    	*str = Trans[i].value;
			    	CLR_BIT(stat, SPECIAL);
			    	break;
			     }
			     i++;
			}
		    }
		    //[BII] end of changes to support &#ddd; escape codes

		    // Did not find the escape code
		    if ( i==NUM_SPECIAL) {
			CLR_BIT(stat, SPECIAL);
//		    	cout.form(UnknownEscape, special);
			continue;
		    }

		    spec_ind = 0;
		}
	    }

	    // If reading a tag
	    if (BIT(stat, READ_COMMAND)) {
		*str = ToLower[*str];
		if ( cmd_ind < MAX_LENGTH ) command[cmd_ind++] = *str;
		continue;
	    } else {
	        // If reading normal text
		if (LegalWordChar[*str]) {
		    if ( word_ind < MAX_LENGTH )
			word[word_ind++] = ToLower[*str];
		} else {
		    // End of a word, put into index
		    if (word_ind != 0) {
			word[word_ind] = '\0';
		    	String str_word = (char *) word;
			
			if (!filter.IsNotAllowed(str_word)) 
			    index._entry.add(word, url);
		    }
		    word_ind = 0;
		}
	    }

	    // Collect title string if within title tag
	    if (BIT(stat, READ_STRING) && !BIT(stat, READ_COMMAND) &&
		!BIT(stat, SPECIAL) && LegalTitleChar[*str] &&
		title_ind < MAX_LENGTH ) {
		if ( *str == '\n' )
		    title[title_ind++] = ' ';
		else 
		    title[title_ind++] = *str;
	    }
	}
    }

    // Set title. Use filename if no title string defined
    if (title_ind == 0)
	strcpy((char *) title, (char *) filename);
    else title[title_ind] = '\0';
    
    index._url(n)->setTitle(title);
}

void Html::parseHtmlCommand(unsigned char* command, short* stat, Url *url,
			    Wordfilter& filter, Index& index, int options)

{
    // Check for title start
    if (strncmp((char *) command, "title", 5) == 0) {
	SET_BIT(*stat, READ_STRING);
	return;
    }

    // title end
    if (strncmp((char *) command, "/title", 6) == 0) {
	CLR_BIT(*stat, READ_STRING);
	return;
    }

    // meta
    if (strncmp((char *) command, "meta", 4) == 0) {
	command += 4;
	while ( *command && isspace(*command) ) command++;

	// meta date
	// Accepts date dd- and date="dd-  "
    	if (strncmp((char *) command, "date", 4) == 0) {
	    long date;
	    command += 4;
	    while ( *command && isspace(*command) ) command++;
	    if ( *command == '=' ) command++;
	    if ( *command == '"' ) command++;
	    date = ParseDate(command);
	    if ( date < 0 ) date = 0;
	    url->setDate(date);
	    return;
	}
    }

    // Comments, index if requested by option
    if ( (options & HOPT_COMMENTS) && strncmp((char *) command, "!", 1) == 0) {
	unsigned char *cp;

	for (cp=command+1; *cp && !LegalWordChar[*cp]; cp++);

	// Find the words and send them into indexer one by one
	while ( *cp ) {

	    // Find the word
	    unsigned char *wstart = cp;
	    for (; *cp && LegalWordChar[*cp]; cp++);
	    if ( *cp ) *(cp++) = '\0';
	    for (; *cp && !LegalWordChar[*cp]; cp++);

	    // Put word into index
	    String str_word = (char *) wstart;	
	    if (!filter.IsNotAllowed(str_word)) 
		index._entry.add(wstart, url);
	}
    }
}

