//
// THIS FILE IS COPYRIGHT 1994, 1995 BY TELENOR R&D, NORWAY
//
//                        ALL RIGHTS RESERVED.
//
// For rights granted to the public see the COPYRIGHT file
//
//
// Url.h	Url class.
//
// Author:  Ken Ronny Schouten
//
// Changes: January 1995 Brd Hfjeld
//				Added date support
//

#ifndef _URL_H
#define _URL_H

#include <string.h>
#include <stream.h>
#include <stdlib.h>
#include <time.h>

extern int strcmp8(const unsigned char *s1, const unsigned char *s2);
void addEscapeCode(unsigned char *s);

#define MAX_URLLENGTH		1024
#define MAX_TITLELENGTH		1024

class Url {
private:
    unsigned char*   _url;	// URL to this document
    unsigned char*   _title;	// Title of this document
    long     	     _date;	// Date for this document (last modified..)
    long             _offset;	// Offset in URL file for this record
public:
    Url(unsigned char* s, unsigned char* t, unsigned long date);
    Url(unsigned char* s);
    Url(const Url& u);
    ~Url();
    operator==(Url& u);
    operator<=(Url& u);

    unsigned char* url();
    unsigned char* title();
    long date();
    void setTitle(unsigned char* title);
    void setDate(unsigned long date);
    long offset();
    void setOffset(long offset);
    void print();
};

inline unsigned char* Url::url()
{
    return _url;
}

inline Url::Url(unsigned char *s, unsigned char *t, unsigned long date)
{
    _url = (unsigned char *) strdup((char *) s);
    _title = (unsigned char *) strdup((char *) t);
    _date = date;
    _offset = 0L; 
}

inline Url::Url(unsigned char *s)
{
    _url = (unsigned char *) strdup((char *) s);
    _title = NULL;
    _date = 0;
    _offset = 0L; 
}

inline Url::Url(const Url& u)
{
    _url = (unsigned char *) strdup((char *) u._url); 
    if (u._title != NULL) {
	_title = (unsigned char *) strdup((char *) u._title);
    }
    _date = u._date;
    _offset = u._offset;
}

inline Url::~Url()
{
    if ( _url ) free(_url); 
    if ( _title ) free(_title); 
}

inline Url::operator==(Url& u)
{
    return strcmp8(_url, u.url()) == 0; 
}

inline Url::operator<=(Url& u)
{
    return strcmp8(_url, u.url()) <= 0; 
}

inline unsigned char* Url::title()
{
    return _title;
}

inline long Url::date()
{
    return _date;
}

inline void Url::setTitle(unsigned char *title)
{
    _title = (unsigned char *) strdup((char *) title);  
}

inline void Url::setDate(unsigned long date)
{
    _date = date;
}

inline long Url::offset()
{
    return _offset;
}

inline void Url::setOffset(long offset)
{ 
    _offset = offset;
}

inline void Url::print()
{
    struct tm *tm;

    tm = gmtime(&_date);
    addEscapeCode(_url);
    addEscapeCode(_title);
    cout << "Url=" << _url << "|Title=" << _title << "|Date=";
    cout        << tm->tm_year+1900
	 << "-" << tm->tm_mon+1
	 << "-" << tm->tm_mday
	 << "-" << tm->tm_hour
	 << "-" << tm->tm_min
	 << "-" << tm->tm_sec;
}

#endif /* _URL_H */
