//
// THIS FILE IS COPYRIGHT 1994, 1995 BY TELENOR R&D, NORWAY
//
//                        ALL RIGHTS RESERVED.
//
// For rights granted to the public see the COPYRIGHT file
//
//
// main.cc	The main routine for the FFW stat program.
//
// Author:  Ken Ronny Schouten
//
// Changes: January 1995 Brd Hfjeld
//				Added no output message
//	    March 1995 Brd Hfjeld
//				Added -t option to check date indices
//

#include <stream.h>
#include <GetOpt.h>
#include <Disk.h>
#include <Entry.h>
#include <lang.h>

#define True (1)
#define False (0)

int main(int argc, char **argv)
{
    int c;
    int cont = 0;
    unsigned char *index_name;
    int did_output = False;

    GetOpt getopt(argc, argv, "i:");
    GetOpt getopt2(argc, argv, "i:df:wt");
    getopt.opterr = 0;
    
    if (argc < 2) {
	cout.form(UsageStat, argv[0]);
	exit(1);
    }

    // Set up time system to be GMT for mktime()
    putenv("TZ=GMT");
    tzset();

    while ((c = getopt()) != EOF) {
	switch (c) {
	case 'i':
	    if (getopt.optarg != "") {
		index_name = (unsigned char *) strdup(getopt.optarg);
	    }
	    cont = 1;
	    break;
	}
    }

    if (cont == 0) {
	cout << NoIndexFiles;
	exit(1);
    }

    Disk *disk= new Disk(index_name, DREAD);
	
    if (disk->error() != 0) {
	switch(disk->error()) {
	case DOPEN:
	    cout.form(OpenFile, index_name);
	    exit(1);
    	case DBAD:
	    cout.form(BadIndex, index_name);
	    exit(1);
	default:
	    cout.form(UnknownError, disk->error(), "Disk::Disk()");
	    exit(1);
	}
    }

    while ((c = getopt2()) != EOF) {
	switch (c) {
	  case 'i':
	    break;
	  case 'd':
	    {
		did_output = True;
		cout.form(BlocksMessage, disk->blocks());
		Entry *entry;
		int first_time = True;
		while (disk->error() == DOK) {
		    entry = disk->nextEntry(first_time);
		    if (entry) {
			first_time = False;
		    	cout << "** "<< entry->word() << endl;
		    	for (Pix i = entry->_urllist.first(); 
			     i != 0;
			     entry->_urllist.next(i)) {
			    cout << "-- ";
		    	    entry->_urllist(i)->print();
			    cout << endl;
		    	}
		    }
		}
		break;
	    }
	  case 'f':
	    {
		// Frequency of a word
		int num = atoi(getopt2.optarg);
		int fre;
		unsigned char *word;
		did_output = True;
		int first_time = True;
		while (disk->error() == DOK) {
		    if ((fre = disk->nextFrequency(word, first_time)) >= num) {
			printf("%6d %s\n",fre,(char *)word);
		    }
		    first_time = False;
		}
		break;
	    }
	  case 'w':
	    {
		// Every word without the url's
		unsigned char *word;
		did_output = True;
		int first_time = True;
		while (disk->error() == DOK) {
		    if ( disk->nextFrequency(word, first_time) > 0 ) {
			first_time = False;
		    	printf("%s\n",(char *)word);
		    }
		}
		break;
	    }

	  case 't':
	    {
		long date;
		unsigned long offset;
		unsigned char urlname[MAX_URLLENGTH];
    		unsigned char title[MAX_TITLELENGTH];
		Url *url;
		long last_date = -1;
	 
		did_output = True;
		while ( disk->error() == DOK ) {
		    disk->GetNextDixEntry(&date, &offset);
		    if ( disk->error() == DOK ) {
			disk->GetURLRecord(offset, &date, urlname, title);
		    	url = new Url(urlname, title, date);
		    	if ( last_date != date ) {
			    cout << date << ", " << offset
				 << " ************************" << endl;
				last_date = date;
		    	}
		    	url->print();
		    	cout << endl;
		    	delete url;
		    }
		}
	    }
	}
    }

    if ( !did_output ) {
	cout << NoOutputOptions;
	cout.form(UsageStat, argv[0]);	
	exit(1);
    }

    exit(0);
}
